@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 1997, 1998, 1999, 2002, 2003, 2004, 2005 Free Software
- * Foundation, Inc.
+ * Copyright (C) 1997, 1998, 1999, 2002, 2003, 2004, 2005, 2006 Free
+ * Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,14 +46,14 @@ gdk_selection_property_get_interp (GdkWindow *requestor,
   /* Note: While we are returning a string, the data may actually be
    * anything and contain zeros. */
   if (data)
-    return scm_take_str (data, length);
+    return scm_take_locale_stringn ((char *) data, length);
   else
     return SCM_BOOL_F;
 }
 
 #define FUNCNAME "gdk-property-change"
 #define BAD_FORMAT(format) \
-  SCM_ASSERT (FALSE, SCM_MAKINUM(format), SCM_ARG6, FUNCNAME)
+  SCM_ASSERT (FALSE, scm_from_int(format), SCM_ARG6, FUNCNAME)
 #define BAD_DATA(data) SCM_ASSERT (FALSE, data, SCM_ARG6, FUNCNAME)
 
 static void
@@ -68,27 +68,27 @@ gdk_property_change_list (GdkWindow *window, GdkAtom property, GdkAtom type,
   switch (format) {
   case 8:
     dc = duchar = g_new (guchar, nelements);
-    for (; SCM_CONSP (data); data = SCM_CDR (data))
+    for (; scm_is_pair (data); data = SCM_CDR (data))
       *dc++ = scm_num2long (SCM_CAR (data), SCM_ARG6, FUNCNAME);
-    if (!SCM_NULLP (data)) BAD_DATA (data);
+    if (!scm_is_null (data)) BAD_DATA (data);
     gdk_property_change (window, property, type, format, mode,
 			 duchar, nelements);
     g_free (duchar);
     break;
   case 16:
     ds = dushort = g_new (gushort, nelements);
-    for (; SCM_CONSP (data); data = SCM_CDR (data))
+    for (; scm_is_pair (data); data = SCM_CDR (data))
       *ds++ = scm_num2long (SCM_CAR (data), SCM_ARG6, FUNCNAME);
-    if (!SCM_NULLP (data)) BAD_DATA (data);
+    if (!scm_is_null (data)) BAD_DATA (data);
     gdk_property_change (window, property, type, format, mode,
 			 (guchar *) dushort, nelements);
     g_free (dushort);
     break;
   case 32:
     dl = dulong = g_new (gulong, nelements);
-    for (; SCM_CONSP (data); data = SCM_CDR (data))
+    for (; scm_is_pair (data); data = SCM_CDR (data))
       *dl++ = scm_num2long (SCM_CAR (data), SCM_ARG6, FUNCNAME);
-    if (!SCM_NULLP (data)) BAD_DATA (data);
+    if (!scm_is_null (data)) BAD_DATA (data);
     gdk_property_change (window, property, type, format, mode,
 			 (guchar *) dulong, nelements);
     g_free (dulong);
@@ -102,60 +102,68 @@ static void
 gdk_property_change_vector (GdkWindow *window, GdkAtom property, GdkAtom type,
 			    gint format, GdkPropMode mode, SCM data)
 {
-  guchar *duchar,*dc;
-  gushort *dushort,*ds;
-  gulong *dulong,*dl;
+  void *c_data;
+  size_t len_data;
+  SCM keep_data;
   int i;
-  int nelements = SCM_VECTOR_LENGTH (data);
-  SCM *velts = SCM_VELTS (data);
+  int nelements;
+
+  if (! (format == 8 || format == 16 || format == 32))
+    BAD_FORMAT (format);
+
+  nelements = scm_c_vector_length (data);
+  len_data = nelements * (format / 8);
+  c_data = scm_malloc (len_data);
+  keep_data = sgtk_make_cblk (c_data, len_data);
+
   switch (format) {
   case 8:
-    dc = duchar = g_new (guchar, nelements);
-    for (i = 0; i < nelements; i++)
-      *dc++ = scm_num2long (velts[i], SCM_ARG6, FUNCNAME);
-    if (!SCM_NULLP (data)) BAD_DATA (data);
-    gdk_property_change (window, property, type, format, mode,
-			 duchar, nelements);
-    g_free (duchar);
+    {
+      guchar *p = (guchar *) c_data;
+      for (i = 0; i < nelements; i++)
+        p[i] = scm_to_uint8 (scm_c_vector_ref (data, i));
+    }
     break;
   case 16:
-    ds = dushort = g_new (gushort, nelements);
-    for (i = 0; i < nelements; i++)
-      *ds++ = scm_num2long (velts[i], SCM_ARG6, FUNCNAME);
-    if (!SCM_NULLP (data)) BAD_DATA (data);
-    gdk_property_change (window, property, type, format, mode,
-			 (guchar *) dushort, nelements);
-    g_free (dushort);
+    {
+      gushort *p = (gushort *) c_data;
+      for (i = 0; i < nelements; i++)
+        p[i] = scm_to_uint16 (scm_c_vector_ref (data, i));
+    }
     break;
-  case 32:
-    dl = dulong = g_new (gulong, nelements);
-    for (i = 0; i < nelements; i++)
-      *dl++ = scm_num2long (velts[i], SCM_ARG6, FUNCNAME);
-    if (!SCM_NULLP (data)) BAD_DATA (data);
-    gdk_property_change (window, property, type, format, mode,
-			 (guchar *) dulong, nelements);
-    g_free (dulong);
+  default: /* 32 */
+    {
+      gulong *p = (gulong *) c_data;
+      for (i = 0; i < nelements; i++)
+        p[i] = scm_to_uint32 (scm_c_vector_ref (data, i));
+    }
     break;
-  default:
-    BAD_FORMAT (format);
   }
+
+  gdk_property_change (window, property, type, format, mode,
+                       c_data, nelements);
+  scm_remember_upto_here_1 (keep_data);
 }
 
 void
 gdk_property_change_interp (GdkWindow *window, GdkAtom property, GdkAtom type,
 			    gint format, GdkPropMode mode, SCM data)
 {
-  if (SCM_NULLP (data))
+  if (scm_is_null (data))
     gdk_property_change (window, property, type, format, mode, NULL, 0);
-  else if (SCM_STRINGP (data)) {
-    if (format != 8) BAD_FORMAT (format);
-    gdk_property_change (window, property, type, format, mode,
-			 (guchar *) SCM_STRING_CHARS (data),
-			 SCM_STRING_LENGTH (data) + 1);
-  }
-  else if (SCM_CONSP (data))
+  else if (scm_is_string (data))
+    {
+      char *c_data;
+      size_t c_len;
+      if (format != 8) BAD_FORMAT (format);
+      c_data = scm_to_locale_stringn (data, &c_len);
+      gdk_property_change (window, property, type, format, mode,
+                           (guchar *) c_data, c_len);
+      free (c_data);
+    }
+  else if (scm_is_pair (data))
     gdk_property_change_list (window, property, type, format, mode, data);
-  else if (SCM_VECTORP (data))
+  else if (scm_is_vector (data))
     gdk_property_change_vector (window, property, type, format, mode, data);
   else BAD_DATA (data);
 }
@@ -197,19 +205,19 @@ sgtk_gdk_gc_values_fill (char *func_name, int argnum,
   mask = 0;
   for (;;)
     {
-      if (! SCM_CONSP (rest))
+      if (! scm_is_pair (rest))
         break;
       key = SCM_CAR (rest);
       rest = SCM_CDR (rest);
 
-      if (! SCM_CONSP (rest))
+      if (! scm_is_pair (rest))
 	scm_misc_error (func_name, "missing argument to keyword ~A",
 			scm_list_1 (key));
       argnum += 2;
       val = SCM_CAR (rest);
       rest = SCM_CDR (rest);
 
-      if (SCM_EQ_P (key, kw_background))
+      if (scm_is_eq (key, kw_background))
 	{
           val = sgtk_color_conversion (val);
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_color_info),
@@ -217,13 +225,13 @@ sgtk_gdk_gc_values_fill (char *func_name, int argnum,
           values->background = * (GdkColor *) sgtk_scm2boxed (val);
 	  mask |= GDK_GC_BACKGROUND;
 	}
-      else if (SCM_EQ_P (key, kw_cap_style))
+      else if (scm_is_eq (key, kw_cap_style))
 	{
           values->cap_style = sgtk_scm2enum (val, &sgtk_gdk_cap_style_info,
                                              argnum, func_name);
 	  mask |= GDK_GC_CAP_STYLE;
 	}
-      else if (SCM_EQ_P (key, kw_clip_mask))
+      else if (scm_is_eq (key, kw_clip_mask))
 	{
           /* this is a bitmap, but the code and docs give it as GdkPixmap */
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_pixmap_info),
@@ -231,30 +239,31 @@ sgtk_gdk_gc_values_fill (char *func_name, int argnum,
           values->clip_mask = (GdkPixmap *) sgtk_scm2boxed (val);
 	  mask |= GDK_GC_CLIP_MASK;
 	}
-      else if (SCM_EQ_P (key, kw_clip_x_origin))
+      else if (scm_is_eq (key, kw_clip_x_origin))
 	{
 	  values->clip_x_origin = scm_num2int (val, argnum, func_name);
 	  mask |= GDK_GC_CLIP_X_ORIGIN;
 	}
-      else if (SCM_EQ_P (key, kw_clip_y_origin))
+      else if (scm_is_eq (key, kw_clip_y_origin))
 	{
 	  values->clip_y_origin = scm_num2int (val, argnum, func_name);
 	  mask |= GDK_GC_CLIP_Y_ORIGIN;
 	}
-      else if (SCM_EQ_P (key, kw_fill))
+      else if (scm_is_eq (key, kw_fill))
 	{
           values->fill = sgtk_scm2enum (val, &sgtk_gdk_fill_info,
                                         argnum, func_name);
 	  mask |= GDK_GC_FILL;
 	}
-      else if (SCM_EQ_P (key, kw_font))
+      else if (scm_is_eq (key, kw_font))
 	{
+          val = sgtk_font_conversion (val);
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_font_info),
                       val, argnum, func_name);
           values->font = (GdkFont *) sgtk_scm2boxed (val);
 	  mask |= GDK_GC_FONT;
 	}
-      else if (SCM_EQ_P (key, kw_foreground))
+      else if (scm_is_eq (key, kw_foreground))
 	{
           val = sgtk_color_conversion (val);
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_color_info),
@@ -262,61 +271,61 @@ sgtk_gdk_gc_values_fill (char *func_name, int argnum,
           values->foreground = * (GdkColor *) sgtk_scm2boxed (val);
 	  mask |= GDK_GC_FOREGROUND;
 	}
-      else if (SCM_EQ_P (key, kw_function))
+      else if (scm_is_eq (key, kw_function))
 	{
           values->function = sgtk_scm2enum (val, &sgtk_gdk_function_info,
                                             argnum, func_name);
 	  mask |= GDK_GC_FUNCTION;
 	}
-      else if (SCM_EQ_P (key, kw_graphics_exposures))
+      else if (scm_is_eq (key, kw_graphics_exposures))
 	{
           values->graphics_exposures = ! SCM_FALSEP (val);
 	  mask |= GDK_GC_EXPOSURES;
 	}
-      else if (SCM_EQ_P (key, kw_join_style))
+      else if (scm_is_eq (key, kw_join_style))
 	{
           values->join_style = sgtk_scm2enum (val, &sgtk_gdk_join_style_info,
                                               argnum, func_name);
 	  mask |= GDK_GC_JOIN_STYLE;
 	}
-      else if (SCM_EQ_P (key, kw_line_style))
+      else if (scm_is_eq (key, kw_line_style))
 	{
           values->line_style = sgtk_scm2enum (val, &sgtk_gdk_line_style_info,
                                               argnum, func_name);
 	  mask |= GDK_GC_LINE_STYLE;
 	}
-      else if (SCM_EQ_P (key, kw_line_width))
+      else if (scm_is_eq (key, kw_line_width))
 	{
 	  values->line_width = scm_num2int (val, argnum, func_name);
 	  mask |= GDK_GC_LINE_WIDTH;
 	}
-      else if (SCM_EQ_P (key, kw_stipple))
+      else if (scm_is_eq (key, kw_stipple))
 	{
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_pixmap_info),
                       val, argnum, func_name);
           values->stipple = (GdkPixmap *) sgtk_scm2boxed (val);
 	  mask |= GDK_GC_STIPPLE;
 	}
-      else if (SCM_EQ_P (key, kw_subwindow_mode))
+      else if (scm_is_eq (key, kw_subwindow_mode))
 	{
           values->subwindow_mode
             = sgtk_scm2enum (val, &sgtk_gdk_subwindow_mode_info,
                              argnum, func_name);
 	  mask |= GDK_GC_SUBWINDOW;
 	}
-      else if (SCM_EQ_P (key, kw_tile))
+      else if (scm_is_eq (key, kw_tile))
 	{
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_pixmap_info),
                       val, argnum, func_name);
           values->tile = (GdkPixmap *) sgtk_scm2boxed (val);
 	  mask |= GDK_GC_TILE;
 	}
-      else if (SCM_EQ_P (key, kw_ts_x_origin))
+      else if (scm_is_eq (key, kw_ts_x_origin))
 	{
 	  values->ts_x_origin = scm_num2int (val, argnum, func_name);
 	  mask |= GDK_GC_TS_X_ORIGIN;
 	}
-      else if (SCM_EQ_P (key, kw_ts_y_origin))
+      else if (scm_is_eq (key, kw_ts_y_origin))
 	{
 	  values->ts_y_origin = scm_num2int (val, argnum, func_name);
 	  mask |= GDK_GC_TS_Y_ORIGIN;
@@ -368,6 +377,7 @@ gdk_window_new_interp (GdkWindow *parent, int width, int height,
   GdkWindow *w;
   SCM key, val;
   unsigned long argnum;
+  SCM keep_lst = SCM_EOL;
 
   attr.event_mask = event_mask;
   attr.width = width;
@@ -379,73 +389,77 @@ gdk_window_new_interp (GdkWindow *parent, int width, int height,
   argnum = 6;
   for (;;)
     {
-      if (! SCM_CONSP (rest))
+      if (! scm_is_pair (rest))
         break;
       key = SCM_CAR (rest);
       rest = SCM_CDR (rest);
 
-      if (! SCM_CONSP (rest))
+      if (! scm_is_pair (rest))
 	scm_misc_error (FUNC_NAME, "missing argument to keyword ~A",
 			scm_list_1 (key));
       argnum += 2;
       val = SCM_CAR (rest);
       rest = SCM_CDR (rest);
 
-      if (SCM_EQ_P (key, kw_colormap))
+      if (scm_is_eq (key, kw_colormap))
 	{
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_colormap_info),
                       val, argnum, FUNC_NAME);
           attr.colormap = (GdkColormap*) sgtk_scm2boxed (val);
 	  mask |= GDK_WA_COLORMAP;
 	}
-      else if (SCM_EQ_P (key, kw_cursor))
+      else if (scm_is_eq (key, kw_cursor))
 	{
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_cursor_info),
                       val, argnum, FUNC_NAME);
           attr.cursor = (GdkCursor*) sgtk_scm2boxed (val);
 	  mask |= GDK_WA_CURSOR;
 	}
-      else if (SCM_EQ_P (key, kw_override_redirect))
+      else if (scm_is_eq (key, kw_override_redirect))
 	{
           attr.override_redirect = ! SCM_FALSEP (val);
 	  mask |= GDK_WA_NOREDIR;
 	}
-      else if (SCM_EQ_P (key, kw_title))
+      else if (scm_is_eq (key, kw_title))
 	{
-	  SCM_VALIDATE_STRING (argnum, val);
-	  attr.title = SCM_STRING_CHARS (val);
+          SCM cstr = sgtk_to_cstr (val);
+          keep_lst = scm_cons (cstr, keep_lst);
+	  attr.title = sgtk_cstr2ptr (cstr, argnum, FUNC_NAME);
 	  mask |= GDK_WA_TITLE;
 	}
-      else if (SCM_EQ_P (key, kw_visual))
+      else if (scm_is_eq (key, kw_visual))
 	{
           SCM_ASSERT (sgtk_valid_boxed (val, &sgtk_gdk_visual_info),
                       val, argnum, FUNC_NAME);
           attr.visual = (GdkVisual*) sgtk_scm2boxed (val);
 	  mask |= GDK_WA_VISUAL;
 	}
-      else if (SCM_EQ_P (key, kw_wmclass))
+      else if (scm_is_eq (key, kw_wmclass))
 	{
-          SCM  val2;
-          if (! SCM_CONSP (rest))
+          SCM  cstr;
+
+          cstr = sgtk_to_cstr (val);
+          keep_lst = scm_cons (cstr, keep_lst);
+          attr.wmclass_name = sgtk_cstr2ptr (cstr, argnum, FUNC_NAME);
+
+          if (! scm_is_pair (rest))
             scm_misc_error (FUNC_NAME, "missing second argument to keyword ~A",
                             scm_list_1 (key));
-          val2 = SCM_CAR (rest);
+          val = SCM_CAR (rest);
           rest = SCM_CDR (rest);
-
-	  SCM_VALIDATE_STRING (argnum, val);
           argnum++;
-	  SCM_VALIDATE_STRING (argnum, val2);
 
-          attr.wmclass_name = SCM_STRING_CHARS (val);
-          attr.wmclass_class = SCM_STRING_CHARS (val);
+          cstr = sgtk_to_cstr (val);
+          keep_lst = scm_cons (cstr, keep_lst);
+          attr.wmclass_class = sgtk_cstr2ptr (cstr, argnum, FUNC_NAME);
 	  mask |= GDK_WA_WMCLASS;
 	}
-      else if (SCM_EQ_P (key, kw_x))
+      else if (scm_is_eq (key, kw_x))
 	{
 	  attr.x = scm_num2short (val, argnum, FUNC_NAME);
 	  mask |= GDK_WA_X;
 	}
-      else if (SCM_EQ_P (key, kw_y))
+      else if (scm_is_eq (key, kw_y))
 	{
 	  attr.y = scm_num2short (val, argnum, FUNC_NAME);
 	  mask |= GDK_WA_Y;
@@ -455,7 +469,8 @@ gdk_window_new_interp (GdkWindow *parent, int width, int height,
     }
 
   w = gdk_window_new (parent, &attr, mask);
-  scm_remember_upto_here (rest);
+  scm_remember_upto_here_1 (rest);
+  scm_remember_upto_here_1 (keep_lst);
   return w;
 }
 #undef FUNC_NAME
@@ -841,7 +856,7 @@ gdk_event_string (GdkEvent *event)
   case GDK_KEY_PRESS:
   case GDK_KEY_RELEASE:
     if (event->key.string != NULL)
-      return scm_mem2string (event->key.string, event->key.length);
+      return scm_from_locale_stringn (event->key.string, event->key.length);
     break;
   default:
     break;
@@ -1063,20 +1078,20 @@ gdk_event_message (GdkEvent *event)
   case GDK_CLIENT_EVENT:
     switch (event->client.data_format) {
     case 8:
-      vector = scm_make_vector (SCM_MAKINUM (20), SCM_BOOL_F);
+      vector = scm_c_make_vector (20, SCM_BOOL_F);
       for (i = 0; i < 20; i++)
-	SCM_WRITABLE_VELTS (vector)[i] = scm_long2num ((long) event->client.data.b[i]);
+	SCM_SIMPLE_VECTOR_SET (vector, i, scm_from_long ((long) event->client.data.b[i]));
       break;
     case 16:
-      vector = scm_make_vector (SCM_MAKINUM (10), SCM_BOOL_F);
+      vector = scm_c_make_vector (10, SCM_BOOL_F);
       for (i = 0; i < 10; i++)
-	SCM_WRITABLE_VELTS (vector)[i] = scm_long2num ((long) event->client.data.s[i]);
+	SCM_SIMPLE_VECTOR_SET (vector, i, scm_from_long ((long) event->client.data.s[i]));
       break;
     case 32:
     default:
-      vector = scm_make_vector (SCM_MAKINUM (5), SCM_BOOL_F);
+      vector = scm_c_make_vector (5, SCM_BOOL_F);
       for (i = 0; i < 5; i++)
-	SCM_WRITABLE_VELTS (vector)[i] = scm_long2num (event->client.data.l[i]);
+	SCM_SIMPLE_VECTOR_SET (vector, i, scm_from_long (event->client.data.l[i]));
     }
     return vector;
   default:
@@ -1110,7 +1125,7 @@ gdk_add_client_message_filter_interp (GdkAtom message_type, SCM filter_proc)
   static SCM client_message_filters = SCM_EOL;
   SCM_ASSERT (SCM_NFALSEP (scm_procedure_p (filter_proc)), filter_proc,
 	      SCM_ARG2, "gdk-add-client-message-filter");
-  if (SCM_NULLP (client_message_filters)) {
+  if (scm_is_null (client_message_filters)) {
     client_message_filters = scm_list_1 (filter_proc);
     scm_permanent_object (client_message_filters);
   }
@@ -1157,29 +1172,29 @@ sgtk_gdk_geometry_fill (const char *func_name, int argnum,
   mask = 0;
   for (;;)
     {
-      if (! SCM_CONSP (rest))
+      if (! scm_is_pair (rest))
         break;
       key = SCM_CAR (rest);
       rest = SCM_CDR (rest);
       argnum++;
 
-      if (SCM_EQ_P (key, kw_pos))
+      if (scm_is_eq (key, kw_pos))
 	{
 	  mask |= GDK_HINT_POS;
 	}
 #if 0   /* gtk 2 */
-      else if (SCM_EQ_P (key, kw_user_pos))
+      else if (scm_is_eq (key, kw_user_pos))
 	{
 	  mask |= GDK_HINT_USER_POS;
 	}
-      else if (SCM_EQ_P (key, kw_user_size))
+      else if (scm_is_eq (key, kw_user_size))
 	{
 	  mask |= GDK_HINT_USER_SIZE;
 	}
 #endif
       else
         {
-          if (! SCM_CONSP (rest))
+          if (! scm_is_pair (rest))
             scm_misc_error (func_name, "missing argument to keyword ~A",
                             scm_list_1 (key));
           val = SCM_CAR (rest);
@@ -1187,7 +1202,7 @@ sgtk_gdk_geometry_fill (const char *func_name, int argnum,
           argnum++;
 
 #if 0  /* gtk 2 */
-          if (SCM_EQ_P (key, kw_win_gravity))
+          if (scm_is_eq (key, kw_win_gravity))
             {
               geom->win_gravity
                 = sgtk_scm2enum (val, &sgtk_gdk_win_gravity_info,
@@ -1197,38 +1212,38 @@ sgtk_gdk_geometry_fill (const char *func_name, int argnum,
           else
 #endif
             {
-              if (! SCM_CONSP (rest))
+              if (! scm_is_pair (rest))
                 scm_misc_error (func_name, "missing second argument to keyword ~A",
                                 scm_list_1 (key));
               val2 = SCM_CAR (rest);
               rest = SCM_CDR (rest);
               argnum++;
 
-              if (SCM_EQ_P (key, kw_aspect))
+              if (scm_is_eq (key, kw_aspect))
                 {
                   geom->min_aspect = scm_num2double (val, argnum-1, func_name);
                   geom->max_aspect = scm_num2double (val2, argnum, func_name);
                   mask |= GDK_HINT_ASPECT;
                 }
-              else if (SCM_EQ_P (key, kw_base_size))
+              else if (scm_is_eq (key, kw_base_size))
                 {
                   geom->base_width  = scm_num2int (val, argnum-1, func_name);
                   geom->base_height = scm_num2int (val2, argnum, func_name);
                   mask |= GDK_HINT_BASE_SIZE;
                 }
-              else if (SCM_EQ_P (key, kw_max_size))
+              else if (scm_is_eq (key, kw_max_size))
                 {
                   geom->max_width  = scm_num2int (val, argnum-1, func_name);
                   geom->max_height = scm_num2int (val2, argnum, func_name);
                   mask |= GDK_HINT_MAX_SIZE;
                 }
-              else if (SCM_EQ_P (key, kw_min_size))
+              else if (scm_is_eq (key, kw_min_size))
                 {
                   geom->min_width  = scm_num2int (val, argnum-1, func_name);
                   geom->min_height = scm_num2int (val2, argnum, func_name);
                   mask |= GDK_HINT_MIN_SIZE;
                 }
-              else if (SCM_EQ_P (key, kw_resize_inc))
+              else if (scm_is_eq (key, kw_resize_inc))
                 {
                   geom->width_inc  = scm_num2int (val, argnum-1, func_name);
                   geom->height_inc = scm_num2int (val2, argnum, func_name);
@@ -1259,7 +1274,7 @@ gdk_window_get_position_interp (GdkWindow *window)
 {
   gint x, y;
   gdk_window_get_position (window, &x, &y);
-  return scm_cons (SCM_MAKINUM (x), SCM_MAKINUM (y));
+  return scm_cons (scm_from_int (x), scm_from_int (y));
 }
 
 SCM
@@ -1267,7 +1282,7 @@ gdk_window_get_root_origin_interp (GdkWindow *window)
 {
   gint x, y;
   gdk_window_get_root_origin (window, &x, &y);
-  return scm_cons (SCM_MAKINUM (x), SCM_MAKINUM (y));
+  return scm_cons (scm_from_int (x), scm_from_int (y));
 }
 
 SCM
@@ -1275,7 +1290,7 @@ gdk_window_get_deskrelative_origin_interp (GdkWindow *window)
 {
   gint x, y;
   gdk_window_get_deskrelative_origin (window, &x, &y);
-  return scm_cons (SCM_MAKINUM (x), SCM_MAKINUM (y));
+  return scm_cons (scm_from_int (x), scm_from_int (y));
 }
 
 
@@ -1285,7 +1300,7 @@ gdk_window_get_size_interp (GdkWindow *window)
   gint width;
   gint height;
   gdk_window_get_size (window, &width, &height);
-  return scm_cons (SCM_MAKINUM (width), SCM_MAKINUM (height));
+  return scm_cons (scm_from_int (width), scm_from_int (height));
 }
 
 SCM
@@ -1294,7 +1309,7 @@ gdk_window_get_origin_interp (GdkWindow *window)
   gint x;
   gint y;
   gdk_window_get_origin (window, &x, &y);
-  return scm_cons (SCM_MAKINUM (x), SCM_MAKINUM (y));
+  return scm_cons (scm_from_int (x), scm_from_int (y));
 }
 
 #define FUNCNAME "gdk-draw-text"
@@ -1302,9 +1317,11 @@ void
 gdk_draw_text_interp (GdkDrawable *drawable, GdkFont *font, GdkGC *gc,
 		      gint x, gint y, SCM text)
 {
-  SCM_ASSERT (SCM_STRINGP (text), text, SCM_ARG6, FUNCNAME);
-  gdk_draw_text (drawable, font, gc, x, y,
-		 SCM_STRING_CHARS (text), SCM_STRING_LENGTH (text));
+  char *c_text;
+  size_t c_len;
+  c_text = scm_to_locale_stringn (text, &c_len);
+  gdk_draw_text (drawable, font, gc, x, y, c_text, c_len);
+  free (c_text);
 }
 #undef FUNCNAME
 
@@ -1314,10 +1331,13 @@ gdk_text_extents_interp (GdkFont *font, SCM text,
 		      gint *lbearing, gint *rbearing, gint *width,
 		      gint *ascent, gint *descent)
 {
-  SCM_ASSERT (SCM_STRINGP (text), text, SCM_ARG2, FUNCNAME);
+  char *c_text;
+  size_t c_len;
+  c_text = scm_to_locale_stringn (text, &c_len);
   /* What about 16-bit fonts ?!?! See spec. */
-  gdk_text_extents (font, SCM_STRING_CHARS (text), SCM_STRING_LENGTH (text),
-		    lbearing, rbearing, width, ascent, descent);
+  gdk_text_extents (font, c_text, c_len,
+                    lbearing, rbearing, width, ascent, descent);
+  free (c_text);
 }
 #undef FUNCNAME
 
@@ -1325,9 +1345,13 @@ gdk_text_extents_interp (GdkFont *font, SCM text,
 gint
 gdk_text_width_interp (GdkFont *font, SCM text)
 {
-  SCM_ASSERT (SCM_STRINGP (text), text, SCM_ARG2, FUNCNAME);
-  return gdk_text_width (font,
-			 SCM_STRING_CHARS (text), SCM_STRING_LENGTH (text));
+  char *c_text;
+  size_t c_len;
+  gint ret;
+  c_text = scm_to_locale_stringn (text, &c_len);
+  ret = gdk_text_width (font, c_text, c_len);
+  free (c_text);
+  return ret;
 }
 #undef FUNCNAME
 
@@ -1335,9 +1359,13 @@ gdk_text_width_interp (GdkFont *font, SCM text)
 gint
 gdk_text_measure_interp (GdkFont *font, SCM text)
 {
-  SCM_ASSERT (SCM_STRINGP (text), text, SCM_ARG2, FUNCNAME);
-  return gdk_text_measure (font,
-			   SCM_STRING_CHARS (text), SCM_STRING_LENGTH (text));
+  char *c_text;
+  size_t c_len;
+  gint ret;
+  c_text = scm_to_locale_stringn (text, &c_len);
+  ret = gdk_text_measure (font, c_text, c_len);
+  free (c_text);
+  return ret;
 }
 #undef FUNCNAME
 
@@ -1345,9 +1373,13 @@ gdk_text_measure_interp (GdkFont *font, SCM text)
 gint
 gdk_text_height_interp (GdkFont *font, SCM text)
 {
-  SCM_ASSERT (SCM_STRINGP (text), text, SCM_ARG2, FUNCNAME);
-  return gdk_text_height (font,
-			  SCM_STRING_CHARS (text), SCM_STRING_LENGTH (text));
+  char *c_text;
+  size_t c_len;
+  gint ret;
+  c_text = scm_to_locale_stringn (text, &c_len);
+  ret = gdk_text_height (font, c_text, c_len);
+  free (c_text);
+  return ret;
 }
 #undef FUNCNAME
 
@@ -1373,7 +1405,7 @@ gdk_mbstowcs_interp (const char *src)
   for (i = 0; i < length && space[i]; i++) ;
   vector = scm_c_make_vector (i, SCM_UNSPECIFIED);
   for (i = 0; i < length && space[i]; i++)
-    SCM_VECTOR_SET (vector, i, scm_long2num (space[i]));
+    SCM_SIMPLE_VECTOR_SET (vector, i, scm_from_long (space[i]));
   g_free (space);
   return vector;
 }
@@ -1397,12 +1429,12 @@ gdk_gc_get_values_interp (GdkGC *gc)
 				     &sgtk_gdk_window_info, TRUE),
 		     sgtk_enum2scm (values.subwindow_mode,
 				     &sgtk_gdk_subwindow_mode_info),
-		     scm_long2num (values.ts_x_origin),
-		     scm_long2num (values.ts_y_origin),
-		     scm_long2num (values.clip_x_origin),
-		     scm_long2num (values.clip_y_origin),
+		     scm_from_long (values.ts_x_origin),
+		     scm_from_long (values.ts_y_origin),
+		     scm_from_long (values.clip_x_origin),
+		     scm_from_long (values.clip_y_origin),
 		     values.graphics_exposures ? SCM_BOOL_T : SCM_BOOL_F,
-		     scm_long2num (values.line_width),
+		     scm_from_long (values.line_width),
 		     sgtk_enum2scm (values.line_style,
 				    &sgtk_gdk_line_style_info),
 		     sgtk_enum2scm (values.cap_style,
@@ -1437,14 +1469,28 @@ gdk_region_get_clipbox_interp (GdkRegion *region)
   return r;
 }
 
+/* Bitmap data as row bits packed into bytes, and each row is padded out to
+   a byte boundary.  */
+static void
+check_bitmap_data_size (const char *func, int count, gint width, gint height)
+{
+  gint bytes_per_row = (width + 7) / 8;
+  gint want_count = bytes_per_row * height;
+  if (count != want_count)
+    scm_misc_error (func,
+		    "bitmap data wrong size, should be ~a bytes",
+		    scm_list_1 (scm_from_int (want_count)));
+}
+
 GdkImage *
 gdk_image_new_bitmap_interp (GdkVisual *visual, guchar data[], int count,
 			     gint width, gint height)
 {
   static const char func_name[] = "gdk-image-new-bitmap";
   gpointer  m_data;
-  if (count * 8 < width * height)
-    scm_misc_error (func_name, "source bitmap is too small", SCM_EOL);
+
+  check_bitmap_data_size ("gdk-bitmap-create-from-data", count, width, height);
+
   m_data = malloc (count);
   if (m_data == NULL)
     scm_memory_error (func_name);
@@ -1456,10 +1502,7 @@ GdkBitmap *
 gdk_bitmap_create_from_data_interp (GdkWindow *window, guchar data[],
 				    int count, gint width, gint height)
 {
-  if (count * 8 < width * height)
-    scm_misc_error ("gdk-bitmap-create-from-data",
-		    "source bitmap is too small",
-		    SCM_EOL);
+  check_bitmap_data_size ("gdk-bitmap-create-from-data", count, width, height);
   return gdk_bitmap_create_from_data (window, data, width, height);
 }
 
@@ -1666,10 +1709,15 @@ sgtk_color_copy (SCM color, void *ptr)
 SCM
 sgtk_string_parse_to_color (SCM obj)
 {
-  if (SCM_STRINGP (obj))
+  if (scm_is_string (obj))
     {
       GdkColor c;
-      if (! gdk_color_parse (SCM_STRING_CHARS (obj), &c))
+      char *c_str;
+      gint ret;
+      c_str = scm_to_locale_string (obj);
+      ret = gdk_color_parse (c_str, &c);
+      free (c_str);
+      if (! ret)
         scm_misc_error ("string->color", "no such color: ~S",
                         scm_list_1 (obj));
       return sgtk_boxed2scm (&c, &sgtk_gdk_color_info, 1);
@@ -1712,15 +1760,15 @@ gdk_colormap_alloc_colors_interp (GdkColormap *colormap, SCM colors,
   /* success flags as list of bools */
   s_success = SCM_EOL;
   for (i = cvec.count-1; i >= 0; i--)
-    s_success = scm_cons (SCM_BOOL (c_success[i]), s_success);
+    s_success = scm_cons (scm_from_bool (c_success[i]), s_success);
   g_free (c_success);
   *s_success_ptr = s_success;
 
   /* put modified "cv" GdkColors back into the "colors" GdkColor objects */
-  if (SCM_VECTORP(colors))
+  if (scm_is_vector (colors))
     {
       for (i = 0; i < cvec.count; i++)
-        memcpy (sgtk_scm2boxed (SCM_VECTOR_REF (colors, i)),
+        memcpy (sgtk_scm2boxed (scm_c_vector_ref (colors, i)),
                 cv+i, sizeof (GdkColor));
     }
   else
@@ -1777,7 +1825,7 @@ gdk_query_depths_interp ()
   gdk_query_depths (&depths,&count);
   for (i = count, list = SCM_EOL;
        i >= 0;
-       list = scm_cons (SCM_MAKINUM (depths[i--]), list))
+       list = scm_cons (scm_from_int (depths[i--]), list))
     ;
   return list;
 }
@@ -1828,12 +1876,16 @@ gdk_text_property_to_text_list_interp (GdkAtom encoding, gint format, SCM text)
   int count;
   gchar **list;
   SCM scmlist;
-  SCM_ASSERT (SCM_STRINGP (text), text, SCM_ARG3, FUNCNAME);
+  char *c_text;
+  size_t c_len;
+
+  c_text = scm_to_locale_stringn (text, &c_len);
   count = gdk_text_property_to_text_list (encoding, format,
-					  (guchar *) SCM_STRING_CHARS (text),
-					  SCM_STRING_LENGTH (text),
-					  &list);
-  if (!count--) return SCM_BOOL_F; /* 0 is not valid; failed */
+                                          (guchar *) c_text, c_len, &list);
+  free (c_text);
+  if (!count--)
+    return SCM_BOOL_F; /* 0 is not valid; failed */
+
   scmlist = scm_makfromstrs (count, list);
   gdk_free_text_list (list);
   return scmlist;
@@ -1849,7 +1901,7 @@ gdk_string_to_compound_text_interp (char *str, GdkAtom *encoding, gint *format,
   ret = gdk_string_to_compound_text (str, encoding, format, &ctext, &length);
   if (ctext != NULL)
     {
-      *textp = scm_mem2string ((char *) ctext, length);
+      *textp = scm_from_locale_stringn ((char *) ctext, length);
       gdk_free_compound_text (ctext);
     }
   else
@@ -1864,15 +1916,12 @@ gdk_property_get_interp (GdkWindow *window, GdkAtom property, GdkAtom type,
 {
   guchar *data;
   gint actual_length;
-  SCM scmdata;
   if (!gdk_property_get (window, property, type, offset, length, pdelete,
 			 actual_property_type, actual_format,
 			 &actual_length, &data))
     return SCM_BOOL_F;
-  /* data is not 0-terminated; gotta copy */
-  scmdata = scm_mem2string ((char *)  data, actual_length);
-  g_free (data);
-  return scmdata;
+
+  return scm_take_locale_stringn ((char *) data, actual_length);
 }
   
 
@@ -1886,5 +1935,7 @@ sgtk_init_gdk_support ()
 #endif /* MKDEP */
 #endif /* SCM_MAGIC_SNARFER */
 
-  scm_c_define ("gdk-priority-events", scm_int2num (GDK_PRIORITY_EVENTS));
+  scm_c_define ("gdk-priority-events", scm_from_int (GDK_PRIORITY_EVENTS));
+  scm_c_define ("gdk-major-version", scm_from_int (gtk_major_version));
+  scm_c_define ("gdk-minor-version", scm_from_int (gtk_minor_version));
 }
