@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997, 1998, 2000, 2001, 2002, 2003, 2004 Free Software
+ * Copyright (C) 1997, 1998, 2000, 2001, 2002, 2003, 2004, 2006 Free Software
  * Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 #include <gtk-1.2/gtk/gtk.h>
 #include <gtk-1.2/gdk/gdkx.h>
 #endif
-#include <guile-gtk.h>
+#include "guile-gtk.h"
 
 /* This whole file is rated XXX. */
 
