@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997, 1998, 1999, 2002, 2003, 2004 Free Software
+ * Copyright (C) 1997, 1998, 1999, 2002, 2003, 2004, 2006 Free Software
  * Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -169,19 +169,15 @@ gtk_style_base_gc (GtkStyle *style, GtkStateType state)
 SCM_KEYWORD (kw_type,  "type");
 SCM_KEYWORD (kw_flags, "flags");
 
-SCM
-gtk_object_query_args_scm (GtkType type)
+static SCM
+args_query_ret (GtkArg *args, guint32 *arg_flags, guint nargs)
 {
-  guint nargs;
-  guint32 *arg_flags = NULL;
-  GtkArg *args;
   SCM res = SCM_EOL, *restail = &res;
   int i;
 
-  args = gtk_object_query_args (type, &arg_flags, &nargs);
   if (args == NULL)
     {
-      if (arg_flags)
+      if (arg_flags != NULL)
 	g_free (arg_flags);
       return SCM_BOOL_F;
     }
@@ -189,13 +185,12 @@ gtk_object_query_args_scm (GtkType type)
   for (i = 0; i < nargs; i++)
     {
       *restail =
-	scm_cons (scm_list_n (scm_makfrom0str (args[i].name),
+	scm_cons (scm_list_5 (scm_from_locale_string (args[i].name),
 			      kw_type,
 			      sgtk_type2scm (args[i].type),
 			      kw_flags,
 			      sgtk_flags2scm (arg_flags[i],
-					      &sgtk_gtk_arg_flags_info),
-			      SCM_UNDEFINED),
+					      &sgtk_gtk_arg_flags_info)),
 		  SCM_EOL);
       restail = SCM_CDRLOC(*restail);
     }
@@ -206,6 +201,26 @@ gtk_object_query_args_scm (GtkType type)
   return res;
 }
 
+SCM
+gtk_object_query_args_interp (GtkType type)
+{
+  guint nargs = 0;
+  guint32 *arg_flags = NULL;
+  GtkArg *args;
+  args = gtk_object_query_args (type, &arg_flags, &nargs);
+  return args_query_ret (args, arg_flags, nargs);
+}
+
+SCM
+gtk_container_query_child_args_interp (GtkType type)
+{
+  guint nargs = 0;
+  guint32 *arg_flags = NULL;
+  GtkArg *args;
+  args = gtk_container_query_child_args (type, &arg_flags, &nargs);
+  return args_query_ret (args, arg_flags, nargs);
+}
+
 int
 gtk_editable_insert_text_scm (GtkEditable *editable,
 			      gchar       *text,
@@ -231,7 +246,79 @@ void gtk_no_free (void *ptr)
 
 SCM
 gtk_selection_data_data (GtkSelectionData* data)
-{ return (data->length >= 0) ? scm_mem2string (data->data, data->length) : SCM_BOOL_F; }
+{
+  if (data->length >= 0)
+    return scm_from_locale_stringn (data->data, data->length);
+  else
+    return SCM_BOOL_F;
+}
+
+
+static GList *
+sgtk_glist_malloc (size_t len)
+{
+  GList *lst = scm_malloc (len * sizeof (GList));
+  size_t i;
+  for (i = 0; i < len; i++)
+    {
+      lst[i].next = &lst[i+1];
+      lst[i].prev = &lst[i-1];
+    }
+  lst[0].prev = NULL;
+  lst[len-1].next = NULL;
+  return lst;
+}
+
+
+/* Explicit code for this isn't nice, but we want to make sure the strings
+   are protected after conversion to cstrs.  This and gdk_drag_begin are the
+   only places a GList is an input, so no burning need for a generic
+   mechanism.  However one thing we can do here with explicit code is
+   disallow an empty list or vector; gtkcombo.c has an assert against
+   that.  */
+void
+gtk_combo_set_popdown_strings_interp (GtkCombo *combo, SCM strings)
+#define FUNC_NAME "gtk-combo-set-popdown-strings"
+{
+  GList *lst;
+  SCM keep;
+
+  if (scm_is_pair (strings))
+    {
+      size_t len = scm_to_size_t (scm_length (strings)), i;
+      lst = sgtk_glist_malloc (len);
+      keep = sgtk_make_cblk (lst, len * sizeof (GList));
+      for (i = 0; i < len; i++, strings = scm_cdr (strings))
+	{
+	  SCM cstr = sgtk_to_cstr (scm_car (strings));
+          keep = scm_cons (cstr, keep);
+          lst[i].data = sgtk_cstr2ptr (cstr, SCM_ARG2, FUNC_NAME);
+	}
+    }
+  else if (scm_is_vector (strings))
+    {
+      size_t len = scm_c_vector_length (strings), i;
+      if (len == 0)
+        goto wrong_type;
+      lst = sgtk_glist_malloc (len);
+      keep = sgtk_make_cblk (lst, len * sizeof (GList));
+      for (i = 0; i < len; i++)
+	{
+	  SCM cstr = sgtk_to_cstr (scm_c_vector_ref (strings, i));
+          keep = scm_cons (cstr, keep);
+          lst[i].data = sgtk_cstr2ptr (cstr, SCM_ARG2, FUNC_NAME);
+	}
+    }
+  else
+    {
+    wrong_type:
+      SCM_WRONG_TYPE_ARG (SCM_ARG2, strings);
+    }
+
+  gtk_combo_set_popdown_strings (combo, lst);
+  scm_remember_upto_here_1 (keep);
+}
+#undef FUNC_NAME
 
 
 /* Same as in gtk-glue.c, could share with that file if it exported this. */
@@ -259,20 +346,15 @@ gtk_container_children_interp (GtkContainer *container)
 SCM
 gtk_rc_get_default_files_interp (void)
 {
-  gchar **a;
-  SCM   ret, *p, c;
+  return scm_makfromstrs (-1, gtk_rc_get_default_files ());
+}
 
-  a = gtk_rc_get_default_files ();
-
-  ret = SCM_EOL;
-  p = &ret;
-  for ( ; *a != NULL; a++)
-    {
-      c = scm_cons (scm_makfrom0str (*a), SCM_EOL);
-      *p = c;
-      p = SCM_CDRLOC (c);
-    }
-  return ret;
+SCM
+gtk_widget_size_request_interp (GtkWidget *widget)
+{
+  GtkRequisition req;
+  gtk_widget_size_request (widget, &req);
+  return scm_cons (scm_from_int (req.width), scm_from_int (req.height));
 }
 
 
@@ -310,16 +392,16 @@ sgtk_init_gtk_support ()
 
   /* Use the library variables so as to indicate what we're running with,
      not GTK_MAJOR_VERSION etc which would be what we compiled against.  */
-  scm_c_define ("gtk-major-version", scm_ulong2num (gtk_major_version));
-  scm_c_define ("gtk-minor-version", scm_ulong2num (gtk_minor_version));
-  scm_c_define ("gtk-micro-version", scm_ulong2num (gtk_micro_version));
-  scm_c_define ("gtk-binary-age",    scm_ulong2num (gtk_binary_age));
-  scm_c_define ("gtk-interface-age", scm_ulong2num (gtk_interface_age));
+  scm_c_define ("gtk-major-version", scm_from_uint (gtk_major_version));
+  scm_c_define ("gtk-minor-version", scm_from_uint (gtk_minor_version));
+  scm_c_define ("gtk-micro-version", scm_from_uint (gtk_micro_version));
+  scm_c_define ("gtk-binary-age",    scm_from_uint (gtk_binary_age));
+  scm_c_define ("gtk-interface-age", scm_from_uint (gtk_interface_age));
 
-  scm_c_define ("gtk-priority-default",  scm_int2num (GTK_PRIORITY_DEFAULT));
-  scm_c_define ("gtk-priority-high",     scm_int2num (GTK_PRIORITY_HIGH));
-  scm_c_define ("gtk-priority-internal", scm_int2num (GTK_PRIORITY_INTERNAL));
-  scm_c_define ("gtk-priority-low",      scm_int2num (GTK_PRIORITY_LOW));
-  scm_c_define ("gtk-priority-redraw",   scm_int2num (GTK_PRIORITY_REDRAW));
-  scm_c_define ("gtk-priority-resize",   scm_int2num (GTK_PRIORITY_RESIZE));
+  scm_c_define ("gtk-priority-default",  scm_from_int (GTK_PRIORITY_DEFAULT));
+  scm_c_define ("gtk-priority-high",     scm_from_int (GTK_PRIORITY_HIGH));
+  scm_c_define ("gtk-priority-internal", scm_from_int (GTK_PRIORITY_INTERNAL));
+  scm_c_define ("gtk-priority-low",      scm_from_int (GTK_PRIORITY_LOW));
+  scm_c_define ("gtk-priority-redraw",   scm_from_int (GTK_PRIORITY_REDRAW));
+  scm_c_define ("gtk-priority-resize",   scm_from_int (GTK_PRIORITY_RESIZE));
 }
