@@ -1,5 +1,5 @@
 /* Threading for guile-gtk
- * Copyright (C) 2000, 2002, 2003 Free Software Foundation
+ * Copyright (C) 2000, 2002, 2003, 2006 Free Software Foundation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,33 +32,19 @@
 
 #include "gtk-threads.h"
 
-#ifndef SCM_USE_COOP_THREADS
-#ifdef USE_COOP_THREADS
+
+/*---------------------------------------------------------------------------*/
+/* Guile 1.6 cooperative threads, ie ./configure --with-threads=coop
+ */
+
+#if USE_COOP_THREADS       /* Guile 1.6 */              \
+  || SCM_USE_COOP_THREADS  /* future Guile 1.8 maybe */ \
+  || SCM_USE_COPT_THREADS  /* future Guile 1.8 maybe */
+#define FOUND 1
+
+#if USE_COOP_THREADS && ! defined (SCM_USE_COOP_THREADS)
 #define SCM_USE_COOP_THREADS 1
-#else
-#define SCM_USE_COOP_THREADS 0
 #endif
-#endif
-
-#ifndef SCM_USE_COPT_THREADS
-#ifdef USE_COPT_THREADS
-#define SCM_USE_COPT_THREADS 1
-#else
-#define SCM_USE_COPT_THREADS 0
-#endif
-#endif
-
-#ifndef SCM_USE_PTHREAD_THREADS
-#ifdef USE_PTHREAD_THREADS
-#define SCM_USE_PTHREAD_THREADS 1
-#else
-#define SCM_USE_PTHREAD_THREADS 0
-#endif
-#endif
-
-#if SCM_USE_COOP_THREADS || SCM_USE_COPT_THREADS || SCM_USE_PTHREAD_THREADS
-
-extern int errno;
 
 #ifdef FD_SET
 #  define SELECT_MASK fd_set
@@ -369,10 +355,15 @@ static void
 g_thread_self_guile_impl (gpointer thread)
 {
 #ifdef SCM_USE_COOP_THREADS
-  * (SCM *) thread = coop_global_curr->handle;
-#endif
+  scm_root_state *rs = coop_global_curr->data;
+  SCM self = rs->handle;
+  * (SCM *) thread = self;
+#else
 #ifdef SCM_USE_COPT_THREADS
   * (SCM *) thread = cur_thread;
+#else
+#error Oops, no g_thread_self_guile_impl code
+#endif
 #endif
 }
 #endif /* HAVE_THREAD_CREATE */
@@ -429,11 +420,44 @@ sgtk_init_threads ()
 #endif
 }
 
-#else /* !USE_THREADS */
+#endif /* coop threads */
 
-#ifdef USE_THREADS
-#error No thread package supported by Guile
-#endif
+
+/*---------------------------------------------------------------------------*/
+/* Guile 1.8 posix pthread threads, ie ./configure --with-threads=pthreads
+ */
+
+#if SCM_USE_PTHREAD_THREADS   /* Guile 1.8 */
+#define FOUND 1
+
+SCM
+sgtk_threads_update ()
+{
+  return SCM_UNSPECIFIED;
+}
+
+void
+sgtk_init_threads ()
+{
+  /* NULL here asks for the default threading, which is posix pthreads on a
+     GNU/Linux system */
+  g_thread_init (NULL);
+}
+
+#endif /* USE_PTHREAD_THREADS */
+
+
+/*---------------------------------------------------------------------------*/
+/* Guile 1.6 or 1.8 null threads, ie ./configure --with-threads=no (or null)
+
+   "! USE_THREADS" is the test in 1.6, but check also the version number,
+   because USE_THREADS is gone altogether in 1.8 too (or at least exists
+   only under SCM_ENABLE_DEPRECATED=1)  */
+
+#if (SCM_MAJOR_VERSION == 1 && SCM_MINOR_VERSION == 6 &&        \
+     ! USE_THREADS)         /* Guile 1.6 */                     \
+  || SCM_USE_NULL_THREADS   /* Guile 1.8 */
+#define FOUND 1
 
 SCM
 sgtk_threads_update ()
@@ -446,4 +470,11 @@ sgtk_init_threads ()
 {
 }
 
-#endif /* !USE_THREADS */
+#endif /* USE_NULL_THREADS */
+
+
+
+/*---------------------------------------------------------------------------*/
+#if ! FOUND
+#error "Oops, Guile threading system recognised"
+#endif
