@@ -1,4 +1,5 @@
-/* Copyright (C) 1997, 1998, 1999, 2003, 2004 Free Software Foundation, Inc.
+/* Copyright (C) 1997, 1998, 1999, 2003, 2004, 2006 Free Software
+ * Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -83,7 +84,7 @@ extern sgtk_boxed_info sgtk_gdk_color_info;
 extern sgtk_boxed_info sgtk_gdk_colormap_info;
 extern sgtk_boxed_info sgtk_gdk_cursor_info;
 extern sgtk_boxed_info sgtk_gdk_font_info;
-#define sgtk_gdk_pixmap_info  sgtk_gdk_window_info
+extern sgtk_boxed_info sgtk_gdk_pixmap_info;
 extern sgtk_boxed_info sgtk_gdk_visual_info;
 extern sgtk_boxed_info sgtk_gdk_window_info;
 extern sgtk_enum_info sgtk_gdk_cap_style_info;
@@ -93,6 +94,7 @@ extern sgtk_enum_info sgtk_gdk_join_style_info;
 extern sgtk_enum_info sgtk_gdk_line_style_info;
 extern sgtk_enum_info sgtk_gdk_subwindow_mode_info;
 extern sgtk_enum_info sgtk_gdk_visual_type_info;
+SCM sgtk_gdk_font_load (SCM font);
 void sgtk_init_gtk_gdk_glue_types (void);
 
 /* gtk-glue.c */
@@ -103,6 +105,15 @@ void sgtk_init_gtk_gtk_glue_types (void);
 void sgtk_init_gtk_gdk_gl_glue_types (void);
 
 
+/* C string objects */
+int sgtk_valid_cstr (SCM obj);
+SCM sgtk_to_cstr (SCM obj);
+char *sgtk_cstr2ptr (SCM obj, unsigned long pos, const char *func_name);
+
+/* C memory objects */
+SCM sgtk_make_cblk (void *p, size_t len);
+
+
 void sgtk_register_type_infos (sgtk_type_info **infos);
 void sgtk_register_type_infos_gtk (GtkTypeInfo **infos);
 sgtk_type_info *sgtk_get_type_info (guint type_seqno);
@@ -131,20 +142,17 @@ void *sgtk_scm2boxed (SCM obj);
 int sgtk_valid_boxed (SCM obj, sgtk_boxed_info*);
 void sgtk_boxed_invalidate (SCM obj);
 
-int sgtk_valid_float (SCM obj);
-gfloat sgtk_scm2float (SCM obj);
-SCM sgtk_float2scm (gfloat f);
-
-int sgtk_valid_double (SCM obj);
-double sgtk_scm2double (SCM obj);
-SCM sgtk_double2scm (double f);
-
 int sgtk_valid_point (SCM obj);
 GdkPoint sgtk_scm2point (SCM obj);
 SCM sgtk_point2scm (GdkPoint p);
 
+struct sgtk_rectangle {
+  int null;
+  GdkRectangle r;
+};
 int sgtk_valid_rect (SCM obj);
 GdkRectangle sgtk_scm2rect (SCM obj);
+struct sgtk_rectangle sgtk_scm2rect_null_ok (SCM obj);
 SCM sgtk_rect2scm (GdkRectangle r);
 
 int sgtk_valid_segment (SCM obj);
@@ -185,11 +193,10 @@ void sgtk_cvec_finish (sgtk_cvec *, SCM obj, SCM (*toscm)(void*), size_t sz);
 typedef struct {
   int count;
   guchar *raw;
-  int must_free;
+  SCM keep;
 } sgtk_raw;
 
 sgtk_raw sgtk_scm2raw (SCM obj, int, char*);
-void sgtk_raw_finish (sgtk_raw* obj);
 
 typedef struct sgtk_protshell sgtk_protshell;
 
@@ -210,12 +217,9 @@ void sgtk_scm2ret (GtkArg *a, SCM obj);
 
 sgtk_object_info *sgtk_find_object_info_from_type (GtkType type);
 sgtk_object_info *sgtk_find_object_info (char *name);
-GtkArg *sgtk_build_args (sgtk_object_info *info, int *n_argsp,
-			 SCM scm_args, SCM protector, char *subr);
 
 SCM sgtk_color_conversion (SCM color);
 SCM sgtk_font_conversion (SCM color);
-SCM sgtk_string_conversion (SCM string);
 
 void sgtk_set_standalone (int flag);
 int sgtk_is_standalone ();
@@ -330,7 +334,8 @@ SCM gdk_gc_get_values_interp (GdkGC *gc);
 
 guint32 gdk_get_leader_window_id ();
 
-SCM gtk_object_query_args_scm (GtkType type);
+SCM gtk_object_query_args_interp (GtkType type);
+SCM gtk_container_query_child_args_interp (GtkType type);
 int gtk_editable_insert_text_scm (GtkEditable *editable,
                                   gchar *text, int position);
 
@@ -346,10 +351,8 @@ gint gdk_text_height_interp (GdkFont *font, SCM text);
 
 /* Gtk stuff that wouldn't be here in an ideal world. */
 
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 0
-typedef int GtkWidgetFlags;
-#endif
-
+SCM gtk_widget_size_request_interp (GtkWidget *widget);
+void gtk_combo_set_popdown_strings_interp (GtkCombo *combo, SCM strings);
 SCM gtk_container_children_interp (GtkContainer *container);
 gchar *gtk_label_get_interp (GtkLabel *label);
 void gtk_menu_popup_interp (GtkMenu *menu,
