@@ -17,7 +17,7 @@
 (load "tests.scm")
 (or (getenv "DISPLAY")
     (begin
-      (display "Cannot run gtk tests without an X display\n")
+      (display "Cannot run gdk tests without an X display\n")
       (exit 77)))
 
 (use-modules (srfi srfi-1)
@@ -57,34 +57,16 @@
        (integer? x)
        (exact? x)))
 
+(define (exact-nonnegative-integer? x)
+  (and (exact-integer? x)
+       (>= x 0)))
+
 (define (exact-positive-integer? x)
   (and (exact-integer? x)
        (positive? x)))
 
 
 
-;;
-;; gdk-drag-begin
-;;
-
-(malloced-steady
- (lambda ()
-   (let* ((targets  '())
-	  (want   #t)
-	  (got    (begin
-		    (gdk-drag-begin sample-win targets)
-		    #t)))
-     (test "gdk-drag-begin" targets want got))))
-
-(malloced-steady
- (lambda ()
-   (let* ((targets  '(STRING))
-	  (want   #t)
-	  (got    (begin
-		    (gdk-drag-begin sample-win targets)
-		    #t)))
-     (test "gdk-drag-begin" targets want got))))
-
 ;;
 ;; gdk-atom-intern
 ;;
@@ -294,6 +276,28 @@
 
       (display "skip gdk-colormap-alloc-colors: no pseudo-color visual\n")))
 
+;;
+;; gdk-drag-begin
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((targets  '())
+	  (want   #t)
+	  (got    (begin
+		    (gdk-drag-begin sample-win targets)
+		    #t)))
+     (test "gdk-drag-begin" targets want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((targets  '(STRING))
+	  (want   #t)
+	  (got    (begin
+		    (gdk-drag-begin sample-win targets)
+		    #t)))
+     (test "gdk-drag-begin" targets want got))))
+
 ;;
 ;; gdk-draw-text
 ;;
@@ -481,6 +485,79 @@
 	  (got  (->bool (apply gdk-gc-new-with-values args))))
      (test "gdk-gc-new-with-values" args want got))))
 
+
+;;
+;; gdk-gc-set-dashes
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  '(5 (6 7 8)))
+	  (want  #t)
+	  (got   (begin
+		   (apply gdk-gc-set-dashes sample-gc args)
+		   #t)))
+     (test "gdk-gc-set-dashes" args want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  '(x (6 7 8)))  ;; bad dash_offset
+	  (want  #f)
+	  (got   (false-if-exception
+		  (begin
+		    (apply gdk-gc-set-dashes sample-gc args)
+		    #t))))
+     (test "gdk-gc-set-dashes" args want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  '(5 (x 1)))
+	  (want  #f)
+	  (got   (false-if-exception
+		  (begin
+		    (apply gdk-gc-set-dashes sample-gc args)
+		    #t))))
+     (test "gdk-gc-set-dashes" args want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  '(5 (1 x)))
+	  (want  #f)
+	  (got   (false-if-exception
+		  (begin
+		    (apply gdk-gc-set-dashes sample-gc args)
+		    #t))))
+     (test "gdk-gc-set-dashes" args want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  '(5 (-128 127)))
+	  (want  #t)
+	  (got   (begin
+		   (apply gdk-gc-set-dashes sample-gc args)
+		   #t)))
+     (test "gdk-gc-set-dashes" args want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  '(5 (1 128))) ;; too big
+	  (want  #f)
+	  (got   (false-if-exception
+		  (begin
+		    (apply gdk-gc-set-dashes sample-gc args)
+		    #t))))
+     (test "gdk-gc-set-dashes" args want got))))
+
+(malloced-steady
+ (lambda ()
+   (let* ((args  '(5 (1 -129))) ;; too big negative
+	  (want  #f)
+	  (got   (false-if-exception
+		  (begin
+		    (apply gdk-gc-set-dashes sample-gc args)
+		    #t))))
+     (test "gdk-gc-set-dashes" args want got))))
+
 ;;
 ;; gdk-gc-set-foreground
 ;;
@@ -529,6 +606,27 @@
 		       (gdk-image-get-pixel image 1 1))))
      (test "gdk-image-new-bitmap" data want got))))
 
+;;
+;; gdk-major-version
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((want #t)
+	  (got  (and (exact-nonnegative-integer? gdk-major-version)
+		     (>= gdk-major-version 1))))
+     (test "gdk-major-version" gdk-major-version want got))))
+
+;;
+;; gdk-minor-version
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((want #t)
+	  (got  (exact-nonnegative-integer? gdk-minor-version)))
+     (test "gdk-minor-version" gdk-minor-version want got))))
+
 ;;
 ;; gdk-pixmap-new
 ;;
@@ -540,6 +638,16 @@
 	  (got    (->bool (apply gdk-pixmap-new args))))
      (test "gdk-pixmap-new" args want got))))
 
+;;
+;; gdk-priority-events
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((want #t)
+	  (got  (exact-integer? gdk-priority-events)))
+     (test "gdk-priority-events" gdk-priority-events want got))))
+
 ;;
 ;; gdk-property-change
 ;;
