@@ -51,6 +51,10 @@
   (and (exact-integer? x)
        (>= x 0)))
 
+(define (exact-positive-integer? x)
+  (and (exact-integer? x)
+       (positive? x)))
+
 ;;
 ;; gtk-adjustment-set-value
 ;;
@@ -338,6 +342,28 @@
 		    #t))))
      (test "gtk-fixed-put" (list x y) want got))))
 
+;;
+;; gtk-item-factory-path-from-widget
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((widget (gtk-label-new "foo"))
+	  (want   #f)
+	  (got    (gtk-item-factory-path-from-widget widget)))
+     (test "gtk-item-factory-path-from-widget" widget want got))))
+
+;; not wrapped yet
+;;
+;; (malloced-steady
+;;  (lambda ()
+;;    (let* ((factory (gtk-item-factory-new 'container "foo" #f))
+;;           ...
+;; 	  (widget  (gtk-item-factory-get-widget factory "bar"))
+;; 	  (want    factory)
+;; 	  (got     (gtk-item-factory-path-from-widget widget)))
+;;      (test "gtk-rc-get-default-files" widget want got))))
+
 ;;
 ;; gtk-rc-get-default-files
 ;;
@@ -456,7 +482,8 @@
 (malloced-steady
  (lambda ()
    (let* ((want #t)
-	  (got  (exact-nonnegative-integer? gtk-major-version)))
+	  (got  (and (exact-nonnegative-integer? gtk-major-version)
+		     (>= gtk-major-version 1))))
      (test "gtk-major-version" #f want got))))
 
 ;;
@@ -503,6 +530,13 @@
 ;; gtk-object-new
 ;;
 
+(malloced-steady
+ (lambda ()
+   (let* ((args  (list 'GtkLabel))
+	  (want  #t)
+	  (got   (gtk-label? (apply gtk-object-new args))))
+     (test "gtk-object-new" args want got))))
+
 (malloced-steady
  (lambda ()
    (let* ((args  (list 'NoSuchType))
@@ -663,6 +697,46 @@
 				(gtk-scrolled-window-get-vadjustment scrolled)))))
      (test "gtk-scrolled-window-new" args want got))))
 
+;;
+;; gtk-signal-disconnect
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((button (gtk-button-new))
+	  (id     (gtk-signal-connect button "clicked" noop)))
+     (gtk-signal-disconnect button id)
+     ;; only checking for no memory leak
+     (test "gtk-signal-disconnect" button #t #t))))
+
+;;
+;; gtk-toolbar-set-style
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((toolbar (gtk-toolbar-new 'horizontal 'icons))
+	  (style   'both)
+	  (want    style)
+	  (got     (begin
+		     (gtk-toolbar-set-style toolbar style)
+		     (gtk-object-get toolbar #:toolbar_style))))
+     (test "gtk-toolbar-set-style" (list toolbar style) want got))))
+
+;;
+;; gtk-widget-size-request
+;;
+
+(malloced-steady
+ (lambda ()
+   (let* ((label (gtk-label-new "foo"))
+	  (want  #t)
+	  (pair  (gtk-widget-size-request label))
+	  (got   (and (pair? pair)
+		      (exact-positive-integer? (car pair))
+		      (exact-positive-integer? (cdr pair)))))
+     (test "gtk-widget-size-request" (list label pair) want got))))
+
 ;;
 ;; signal call font
 ;;
